@@ -14,6 +14,11 @@ char *janus_log_global_prefix = NULL;
 int lock_debug = 0;
 int refcount_debug = 0;
 
+/* This is to avoid linking with openSSL */
+int RAND_bytes(uint8_t *key, int len) {
+	return 0;
+}
+
 int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
 	/* Since we're fuzzing SDP, and that in our case SDP always comes
 	 * from a Jansson call, this will need to be a valid string */
