@@ -60,9 +60,14 @@ var videoenabled = false;
 
 var doSimulcast = (getQueryStringValue("simulcast") === "yes" || getQueryStringValue("simulcast") === "true");
 var doSimulcast2 = (getQueryStringValue("simulcast2") === "yes" || getQueryStringValue("simulcast2") === "true");
+var doSvc = getQueryStringValue("svc");
+if(doSvc === "")
+	doSvc = null;
 var acodec = (getQueryStringValue("acodec") !== "" ? getQueryStringValue("acodec") : null);
 var vcodec = (getQueryStringValue("vcodec") !== "" ? getQueryStringValue("vcodec") : null);
 var vprofile = (getQueryStringValue("vprofile") !== "" ? getQueryStringValue("vprofile") : null);
+var doDtx = (getQueryStringValue("dtx") === "yes" || getQueryStringValue("dtx") === "true");
+var doOpusred = (getQueryStringValue("opusred") === "yes" || getQueryStringValue("opusred") === "true");
 var simulcastStarted = false;
 
 $(document).ready(function() {
@@ -111,6 +116,9 @@ $(document).ready(function() {
 									// profile as well (e.g., ?vprofile=2 for VP9, or ?vprofile=42e01f for H.264)
 									if(vprofile)
 										body["videoprofile"] = vprofile;
+									// We can force RED for audio too, if supported by the browser
+									if(doOpusred)
+										body["opusred"] = true;
 									Janus.debug("Sending message:", body);
 									echotest.send({ message: body });
 									Janus.debug("Trying a createOffer too (audio/video sendrecv)");
@@ -123,6 +131,14 @@ $(document).ready(function() {
 											// the following 'simulcast' property to pass to janus.js to true
 											simulcast: doSimulcast,
 											simulcast2: doSimulcast2,
+											svc: (vcodec === 'av1' && doSvc) ? doSvc : null,
+											customizeSdp: function(jsep) {
+												// If DTX is enabled, munge the SDP
+												if(doDtx) {
+													jsep.sdp = jsep.sdp
+														.replace("useinbandfec=1", "useinbandfec=1;usedtx=1")
+												}
+											},
 											success: function(jsep) {
 												Janus.debug("Got SDP!", jsep);
 												echotest.send({ message: body, jsep: jsep });
@@ -214,7 +230,7 @@ $(document).ready(function() {
 									if((substream !== null && substream !== undefined) || (temporal !== null && temporal !== undefined)) {
 										if(!simulcastStarted) {
 											simulcastStarted = true;
-											addSimulcastButtons(msg["videocodec"] === "vp8" || msg["videocodec"] === "h264");
+											addSimulcastButtons(msg["videocodec"] === "vp8");
 										}
 										// We just received notice that there's been a switch, update the buttons
 										updateSimulcastButtons(substream, temporal);
@@ -446,6 +462,12 @@ function addSimulcastButtons(temporal) {
 		'		</div>' +
 		'	</div>' +
 		'</div>');
+	if(Janus.webRTCAdapter.browserDetails.browser !== "firefox") {
+		// Chromium-based browsers only have two temporal layers
+		$('#tl-2').remove();
+		$('#tl-1').css('width', '50%');
+		$('#tl-0').css('width', '50%');
+	}
 	// Enable the simulcast selection buttons
 	$('#sl-0').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
