@@ -5,14 +5,15 @@
  * \details  Implementation of a simple buffered logger designed to remove
  * I/O wait from threads that may be sensitive to such delays. Buffers are
  * saved and reused to reduce allocation calls. The logger output can then
- * be printed to stdout and/or a log file.
+ * be printed to stdout and/or a log file. If external loggers are added
+ * to the core, the logger output is passed to those as well.
  *
  * \ingroup core
  * \ref core
  */
 
-#ifndef _JANUS_LOG_H
-#define _JANUS_LOG_H
+#ifndef JANUS_LOG_H
+#define JANUS_LOG_H
 
 #include <stdio.h>
 #include <glib.h>
@@ -31,6 +32,9 @@ void janus_vprintf(const char *format, ...) G_GNUC_PRINTF(1, 2);
 * @param logfile Log file to save the output to, if any
 * @returns 0 in case of success, a negative integer otherwise */
 int janus_log_init(gboolean daemon, gboolean console, const char *logfile);
+/*! \brief Method to add a list of external loggers to the log management
+ * @param loggers Hash table of external loggers registered in the core */
+void janus_log_set_loggers(GHashTable *loggers);
 /*! \brief Log destruction */
 void janus_log_destroy(void);
 
