@@ -188,6 +188,23 @@ janus_plugin *create(void) {
 	return &janus_echotest_plugin;
 }
 
+/* Parameter validation */
+static struct janus_json_parameter request_parameters[] = {
+	{"audio", JANUS_JSON_BOOL, 0},
+	{"video", JANUS_JSON_BOOL, 0},
+	{"bitrate", JSON_INTEGER, JANUS_JSON_PARAM_POSITIVE},
+	{"record", JANUS_JSON_BOOL, 0},
+	{"filename", JSON_STRING, 0},
+	{"substream", JSON_INTEGER, JANUS_JSON_PARAM_POSITIVE},
+	{"temporal", JSON_INTEGER, JANUS_JSON_PARAM_POSITIVE},
+	{"fallback", JSON_INTEGER, JANUS_JSON_PARAM_POSITIVE},
+	{"record", JANUS_JSON_BOOL, 0},
+	{"filename", JSON_STRING, 0},
+	{"audiocodec", JSON_STRING, 0},
+	{"videocodec", JSON_STRING, 0},
+	{"videoprofile", JSON_STRING, 0},
+	{"opusred", JANUS_JSON_BOOL, 0},
+};
 
 /* Useful stuff */
 static volatile gint initialized = 0, stopping = 0;
@@ -216,6 +233,7 @@ typedef struct janus_echotest_session {
 	janus_audiocodec acodec;/* Codec used for audio, if available */
 	janus_videocodec vcodec;/* Codec used for video, if available */
 	char *vfmtp;
+	int opusred_pt;
 	uint32_t bitrate, peer_bitrate;
 	janus_rtp_switching_context context;
 	uint32_t ssrc[3];		/* Only needed in case VP8 (or H.264) simulcasting is involved */
@@ -461,8 +479,11 @@ json_t *janus_echotest_query_session(janus_plugin_session *handle) {
 	json_t *info = json_object();
 	json_object_set_new(info, "audio_active", session->audio_active ? json_true() : json_false());
 	json_object_set_new(info, "video_active", session->video_active ? json_true() : json_false());
-	if(session->acodec != JANUS_AUDIOCODEC_NONE)
+	if(session->acodec != JANUS_AUDIOCODEC_NONE) {
 		json_object_set_new(info, "audio_codec", json_string(janus_audiocodec_name(session->acodec)));
+		if(session->opusred_pt)
+			json_object_set_new(info, "audio_red", json_true());
+	}
 	if(session->vcodec != JANUS_VIDEOCODEC_NONE)
 		json_object_set_new(info, "video_codec", json_string(janus_videocodec_name(session->vcodec)));
 	json_object_set_new(info, "bitrate", json_integer(session->bitrate));
@@ -661,10 +682,12 @@ void janus_echotest_incoming_data(janus_plugin_session *handle, janus_plugin_dat
 		if(packet->buffer == NULL || packet->length == 0)
 			return;
 		char *label = packet->label;
+		char *protocol = packet->protocol;
 		char *buf = packet->buffer;
 		uint16_t len = packet->length;
 		if(packet->binary) {
-			JANUS_LOG(LOG_VERB, "Got a binary DataChannel message (label=%s, %d bytes) to bounce back\n", label, len);
+			JANUS_LOG(LOG_VERB, "Got a binary DataChannel message (label=%s, protocol=%s, %d bytes) to bounce back\n",
+				label, protocol, len);
 			/* Save the frame if we're recording */
 			janus_recorder_save_frame(session->drc, buf, len);
 			/* Binary data, shoot back as it is */
@@ -675,7 +698,8 @@ void janus_echotest_incoming_data(janus_plugin_session *handle, janus_plugin_dat
 		char *text = g_malloc(len+1);
 		memcpy(text, buf, len);
 		*(text+len) = '\0';
-		JANUS_LOG(LOG_VERB, "Got a DataChannel message (label=%s, %zu bytes) to bounce back: %s\n", label, strlen(text), text);
+		JANUS_LOG(LOG_VERB, "Got a DataChannel message (label=%s, protocol=%s, %zu bytes) to bounce back: %s\n",
+			label, protocol, strlen(text), text);
 		/* Save the frame if we're recording */
 		janus_recorder_save_frame(session->drc, text, strlen(text));
 		/* We send back the same text with a custom prefix */
@@ -686,6 +710,7 @@ void janus_echotest_incoming_data(janus_plugin_session *handle, janus_plugin_dat
 		/* Prepare the packet and send it back */
 		janus_plugin_data r = {
 			.label = label,
+			.protocol = protocol,
 			.binary = FALSE,
 			.buffer = reply,
 			.length = strlen(reply)
@@ -811,6 +836,7 @@ static void janus_echotest_hangup_media_internal(janus_plugin_session *handle) {
 	session->vcodec = JANUS_VIDEOCODEC_NONE;
 	g_free(session->vfmtp);
 	session->vfmtp = NULL;
+	session->opusred_pt = -1;
 	session->e2ee = FALSE;
 	session->bitrate = 0;
 	session->peer_bitrate = 0;
@@ -871,6 +897,11 @@ static void *janus_echotest_handler(void *data) {
 			goto error;
 		}
 		/* Parse request */
+		JANUS_VALIDATE_JSON_OBJECT(root, request_parameters,
+			error_code, error_cause, TRUE,
+			0, JANUS_ECHOTEST_ERROR_INVALID_ELEMENT);
+		if(error_code != 0)
+			goto error;
 		const char *msg_sdp_type = json_string_value(json_object_get(msg->jsep, "type"));
 		const char *msg_sdp = json_string_value(json_object_get(msg->jsep, "sdp"));
 		json_t *msg_simulcast = json_object_get(msg->jsep, "simulcast");
@@ -886,82 +917,29 @@ static void *janus_echotest_handler(void *data) {
 		if(json_is_true(msg_e2ee))
 			session->e2ee = TRUE;
 		json_t *audio = json_object_get(root, "audio");
-		if(audio && !json_is_boolean(audio)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (audio should be a boolean)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (audio should be a boolean)");
-			goto error;
-		}
 		json_t *video = json_object_get(root, "video");
-		if(video && !json_is_boolean(video)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (video should be a boolean)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (video should be a boolean)");
-			goto error;
-		}
 		json_t *bitrate = json_object_get(root, "bitrate");
-		if(bitrate && (!json_is_integer(bitrate) || json_integer_value(bitrate) < 0)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (bitrate should be a positive integer)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (bitrate should be a positive integer)");
-			goto error;
-		}
 		json_t *substream = json_object_get(root, "substream");
-		if(substream && (!json_is_integer(substream) || json_integer_value(substream) < 0 || json_integer_value(substream) > 2)) {
+		if(substream && json_integer_value(substream) > 2) {
 			JANUS_LOG(LOG_ERR, "Invalid element (substream should be 0, 1 or 2)\n");
 			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
 			g_snprintf(error_cause, 512, "Invalid value (substream should be 0, 1 or 2)");
 			goto error;
 		}
 		json_t *temporal = json_object_get(root, "temporal");
-		if(temporal && (!json_is_integer(temporal) || json_integer_value(temporal) < 0 || json_integer_value(temporal) > 2)) {
+		if(temporal && json_integer_value(temporal) > 2) {
 			JANUS_LOG(LOG_ERR, "Invalid element (temporal should be 0, 1 or 2)\n");
 			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
 			g_snprintf(error_cause, 512, "Invalid value (temporal should be 0, 1 or 2)");
 			goto error;
 		}
 		json_t *fallback = json_object_get(root, "fallback");
-		if(fallback && (!json_is_integer(fallback) || json_integer_value(fallback) < 0)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (fallback should be a positive integer)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (fallback should be a positive integer)");
-			goto error;
-		}
 		json_t *record = json_object_get(root, "record");
-		if(record && !json_is_boolean(record)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (record should be a boolean)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (record should be a boolean)");
-			goto error;
-		}
 		json_t *recfile = json_object_get(root, "filename");
-		if(recfile && !json_is_string(recfile)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (filename should be a string)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (filename should be a string)");
-			goto error;
-		}
 		json_t *audiocodec = json_object_get(root, "audiocodec");
-		if(audiocodec && !json_is_string(audiocodec)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (audiocodec should be a string)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (audiocodec should be a string)");
-			goto error;
-		}
 		json_t *videocodec = json_object_get(root, "videocodec");
-		if(videocodec && !json_is_string(videocodec)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (videocodec should be a string)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (videocodec should be a string)");
-			goto error;
-		}
 		json_t *videoprofile = json_object_get(root, "videoprofile");
-		if(videoprofile && !json_is_string(videoprofile)) {
-			JANUS_LOG(LOG_ERR, "Invalid element (videoprofile should be a string)\n");
-			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Invalid value (videoprofile should be a string)");
-			goto error;
-		}
+		json_t *opusred = json_object_get(root, "opusred");
 		/* Enforce request */
 		if(audio) {
 			session->audio_active = json_is_true(audio);
@@ -989,8 +967,10 @@ static void *janus_echotest_handler(void *data) {
 		}
 		if(substream) {
 			session->sim_context.substream_target = json_integer_value(substream);
-			JANUS_LOG(LOG_VERB, "Setting video SSRC to let through (simulcast): %"SCNu32" (index %d, was %d)\n",
-				session->ssrc[session->sim_context.substream], session->sim_context.substream_target, session->sim_context.substream);
+			if(session->sim_context.substream_target >= 0 && session->sim_context.substream_target <= 2) {
+				JANUS_LOG(LOG_VERB, "Setting video SSRC to let through (simulcast): %"SCNu32" (index %d, was %d)\n",
+					session->ssrc[session->sim_context.substream_target], session->sim_context.substream_target, session->sim_context.substream);
+			}
 			if(session->sim_context.substream_target == session->sim_context.substream) {
 				/* No need to do anything, we're already getting the right substream, so notify the user */
 				json_t *event = json_object();
@@ -1032,10 +1012,10 @@ static void *janus_echotest_handler(void *data) {
 			session->has_data = (strstr(msg_sdp, "DTLS/SCTP") != NULL);
 		}
 
-		if(!audio && !video && !videocodec && !videoprofile && !bitrate && !substream && !temporal && !fallback && !record && !msg_sdp) {
-			JANUS_LOG(LOG_ERR, "No supported attributes (audio, video, videocodec, videoprofile, bitrate, substream, temporal, fallback, record, jsep) found\n");
+		if(!audio && !video && !videocodec && !videoprofile && !opusred && !bitrate && !substream && !temporal && !fallback && !record && !msg_sdp) {
+			JANUS_LOG(LOG_ERR, "No supported attributes found\n");
 			error_code = JANUS_ECHOTEST_ERROR_INVALID_ELEMENT;
-			g_snprintf(error_cause, 512, "Message error: no supported attributes (audio, video, videocodec, videoprofile, bitrate, simulcast, temporal, fallback, record, jsep) found");
+			g_snprintf(error_cause, 512, "Message error: no supported attributes found");
 			goto error;
 		}
 
@@ -1059,8 +1039,10 @@ static void *janus_echotest_handler(void *data) {
 				g_snprintf(error_cause, 512, "Error parsing offer: %s", error_str);
 				goto error;
 			}
-			/* Check if we need to negotiate Opus FEC */
-			gboolean opus_fec = FALSE;
+			/* Check if we need to negotiate Opus FEC and/or DTX */
+			gboolean opus_fec = FALSE, opus_dtx = FALSE;
+			char custom_fmtp[256];
+			custom_fmtp[0] = '\0';
 			GList *temp = offer->m_lines;
 			while(temp) {
 				/* Which media are available? */
@@ -1071,9 +1053,31 @@ static void *janus_echotest_handler(void *data) {
 					while(ma) {
 						janus_sdp_attribute *a = (janus_sdp_attribute *)ma->data;
 						if(a->value) {
-							if(m->type == JANUS_SDP_AUDIO && !strcasecmp(a->name, "fmtp") &&
-									strstr(a->value, "useinbandfec=1")) {
-								opus_fec = TRUE;
+							if(m->type == JANUS_SDP_AUDIO && !strcasecmp(a->name, "fmtp")) {
+								if(strstr(a->value, "useinbandfec=1")) {
+									opus_fec = TRUE;
+									if(strlen(custom_fmtp) == 0) {
+										g_snprintf(custom_fmtp, sizeof(custom_fmtp), "useinbandfec=1");
+									} else {
+										g_strlcat(custom_fmtp, ";useinbandfec=1", sizeof(custom_fmtp));
+									}
+								}
+								if(strstr(a->value, "usedtx=1")) {
+									opus_dtx = TRUE;
+									if(strlen(custom_fmtp) == 0) {
+										g_snprintf(custom_fmtp, sizeof(custom_fmtp), "usedtx=1");
+									} else {
+										g_strlcat(custom_fmtp, ";usedtx=1", sizeof(custom_fmtp));
+									}
+								}
+								if(strstr(a->value, "stereo=1")) {
+									opus_dtx = TRUE;
+									if(strlen(custom_fmtp) == 0) {
+										g_snprintf(custom_fmtp, sizeof(custom_fmtp), "usedtx=1");
+									} else {
+										g_strlcat(custom_fmtp, ";usedtx=1", sizeof(custom_fmtp));
+									}
+								}
 							}
 						}
 						ma = ma->next;
@@ -1083,7 +1087,8 @@ static void *janus_echotest_handler(void *data) {
 			}
 			janus_sdp *answer = janus_sdp_generate_answer(offer,
 				JANUS_SDP_OA_AUDIO_CODEC, json_string_value(audiocodec),
-				JANUS_SDP_OA_AUDIO_FMTP, opus_fec ? "useinbandfec=1" : NULL,
+				JANUS_SDP_OA_AUDIO_FMTP, (opus_fec || opus_dtx ? custom_fmtp : NULL),
+				JANUS_SDP_OA_ACCEPT_OPUSRED, json_is_true(opusred),
 				JANUS_SDP_OA_VIDEO_CODEC, json_string_value(videocodec),
 				JANUS_SDP_OA_VP9_PROFILE, json_string_value(videoprofile),
 				JANUS_SDP_OA_H264_PROFILE, json_string_value(videoprofile),
@@ -1093,6 +1098,7 @@ static void *janus_echotest_handler(void *data) {
 				JANUS_SDP_OA_ACCEPT_EXTMAP, JANUS_RTP_EXTMAP_AUDIO_LEVEL,
 				JANUS_SDP_OA_ACCEPT_EXTMAP, JANUS_RTP_EXTMAP_VIDEO_ORIENTATION,
 				JANUS_SDP_OA_ACCEPT_EXTMAP, JANUS_RTP_EXTMAP_TRANSPORT_WIDE_CC,
+				JANUS_SDP_OA_ACCEPT_EXTMAP, JANUS_RTP_EXTMAP_DEPENDENCY_DESC,
 				JANUS_SDP_OA_DONE);
 			/* If we ended up sendonly, switch to inactive (as we don't really send anything ourselves) */
 			janus_sdp_mline *m = janus_sdp_mline_find(answer, JANUS_SDP_AUDIO);
@@ -1126,6 +1132,8 @@ static void *janus_echotest_handler(void *data) {
 				if(vfmtp != NULL)
 					session->vfmtp = g_strdup(vfmtp);
 			}
+			if(json_is_true(opusred))
+				session->opusred_pt = janus_sdp_get_opusred_pt(answer);
 			/* Done */
 			char *sdp = janus_sdp_write(answer);
 			janus_sdp_destroy(offer);
@@ -1176,6 +1184,9 @@ static void *janus_echotest_handler(void *data) {
 							JANUS_LOG(LOG_ERR, "Couldn't open an audio recording file for this EchoTest user!\n");
 						}
 					}
+					/* If RED is in use, take note of it */
+					if(session->opusred_pt > 0)
+						janus_recorder_opusred(rc, session->opusred_pt);
 					/* If media is encrypted, mark it in the recording */
 					if(session->e2ee)
 						janus_recorder_encrypted(rc);
