@@ -374,6 +374,7 @@ typedef struct janus_videocall_session {
 	gboolean video_active;
 	janus_audiocodec acodec;/* Codec used for audio, if available */
 	janus_videocodec vcodec;/* Codec used for video, if available */
+	int opusred_pt;
 	uint32_t bitrate, peer_bitrate;
 	guint16 slowlink_count;
 	struct janus_videocall_session *peer;
@@ -655,8 +656,11 @@ json_t *janus_videocall_query_session(janus_plugin_session *handle) {
 		json_object_set_new(info, "peer", peer->username ? json_string(peer->username) : NULL);
 		json_object_set_new(info, "audio_active", session->audio_active ? json_true() : json_false());
 		json_object_set_new(info, "video_active", session->video_active ? json_true() : json_false());
-		if(session->acodec != JANUS_AUDIOCODEC_NONE)
+		if(session->acodec != JANUS_AUDIOCODEC_NONE) {
 			json_object_set_new(info, "audio_codec", json_string(janus_audiocodec_name(session->acodec)));
+			if(session->opusred_pt)
+				json_object_set_new(info, "audio_red", json_true());
+		}
 		if(session->vcodec != JANUS_VIDEOCODEC_NONE)
 			json_object_set_new(info, "video_codec", json_string(janus_videocodec_name(session->vcodec)));
 		json_object_set_new(info, "video_active", session->video_active ? json_true() : json_false());
@@ -786,7 +790,7 @@ void janus_videocall_incoming_rtp(janus_plugin_session *handle, janus_plugin_rtp
 				json_t *result = json_object();
 				json_object_set_new(result, "event", json_string("simulcast"));
 				json_object_set_new(result, "videocodec", json_string(janus_videocodec_name(session->vcodec)));
-				json_object_set_new(result, "substream", json_integer(session->sim_context.substream));
+				json_object_set_new(result, "substream", json_integer(peer->sim_context.substream));
 				json_object_set_new(event, "result", result);
 				gateway->push_event(peer->handle, &janus_videocall_plugin, NULL, event, NULL);
 				json_decref(event);
@@ -798,7 +802,7 @@ void janus_videocall_incoming_rtp(janus_plugin_session *handle, janus_plugin_rtp
 				json_t *result = json_object();
 				json_object_set_new(result, "event", json_string("simulcast"));
 				json_object_set_new(result, "videocodec", json_string(janus_videocodec_name(session->vcodec)));
-				json_object_set_new(result, "temporal", json_integer(session->sim_context.templayer));
+				json_object_set_new(result, "temporal", json_integer(peer->sim_context.templayer));
 				json_object_set_new(event, "result", result);
 				gateway->push_event(peer->handle, &janus_videocall_plugin, NULL, event, NULL);
 				json_decref(event);
@@ -1030,6 +1034,7 @@ static void janus_videocall_hangup_media_internal(janus_plugin_session *handle)
 	session->video_active = TRUE;
 	session->acodec = JANUS_AUDIOCODEC_NONE;
 	session->vcodec = JANUS_VIDEOCODEC_NONE;
+	session->opusred_pt = 0;
 	session->bitrate = 0;
 	session->peer_bitrate = 0;
 	session->e2ee = FALSE;
@@ -1357,21 +1362,13 @@ static void *janus_videocall_handler(void *data) {
 			/* Check if this user will simulcast */
 			json_t *msg_simulcast = json_object_get(msg->jsep, "simulcast");
 			if(msg_simulcast && janus_get_codec_pt(msg_sdp, "vp8") > 0) {
-				JANUS_LOG(LOG_VERB, "VideoCall callee (%s) is going to do simulcasting\n", session->username);
-				session->ssrc[0] = json_integer_value(json_object_get(msg_simulcast, "ssrc-0"));
-				session->ssrc[1] = json_integer_value(json_object_get(msg_simulcast, "ssrc-1"));
-				session->ssrc[2] = json_integer_value(json_object_get(msg_simulcast, "ssrc-2"));
+				JANUS_LOG(LOG_VERB, "VideoCall callee (%s) cannot do simulcast.\n", session->username);
 			} else {
 				int i=0;
 				for(i=0; i<3; i++) {
 					session->ssrc[i] = 0;
 					g_free(session->rid[0]);
 					session->rid[0] = NULL;
-					if(peer) {
-						peer->ssrc[i] = 0;
-						g_free(peer->rid[0]);
-						peer->rid[0] = NULL;
-					}
 				}
 			}
 			/* Check which codecs we ended up using */
@@ -1379,6 +1376,9 @@ static void *janus_videocall_handler(void *data) {
 			janus_sdp_find_first_codecs(answer, &acodec, &vcodec);
 			session->acodec = janus_audiocodec_from_name(acodec);
 			session->vcodec = janus_videocodec_from_name(vcodec);
+			session->opusred_pt = janus_sdp_get_opusred_pt(answer);
+			if(peer)
+				peer->opusred_pt = session->opusred_pt;
 			if(session->acodec == JANUS_AUDIOCODEC_NONE) {
 				session->has_audio = FALSE;
 				if(peer)
@@ -1490,8 +1490,10 @@ static void *janus_videocall_handler(void *data) {
 			}
 			if(substream) {
 				session->sim_context.substream_target = json_integer_value(substream);
-				JANUS_LOG(LOG_VERB, "Setting video SSRC to let through (simulcast): %"SCNu32" (index %d, was %d)\n",
-					session->ssrc[session->sim_context.substream], session->sim_context.substream_target, session->sim_context.substream);
+				if(session->sim_context.substream_target >= 0 && session->sim_context.substream_target <= 2) {
+					JANUS_LOG(LOG_VERB, "Setting video SSRC to let through (simulcast): %"SCNu32" (index %d, was %d)\n",
+						session->ssrc[session->sim_context.substream_target], session->sim_context.substream_target, session->sim_context.substream);
+				}
 				if(session->sim_context.substream_target == session->sim_context.substream) {
 					/* No need to do anything, we're already getting the right substream, so notify the user */
 					json_t *event = json_object();
@@ -1573,6 +1575,9 @@ static void *janus_videocall_handler(void *data) {
 								JANUS_LOG(LOG_ERR, "Couldn't open an audio recording file for this VideoCall user!\n");
 							}
 						}
+						/* If RED is in use, take note of it */
+						if(session->opusred_pt > 0)
+							janus_recorder_opusred(rc, session->opusred_pt);
 						/* If media is encrypted, mark it in the recording */
 						if(session->e2ee)
 							janus_recorder_encrypted(rc);
