@@ -19,144 +19,61 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <libavcodec/avcodec.h>
-#include <libavformat/avformat.h>
-
+#include "pp-avformat.h"
 #include "pp-av1.h"
 #include "../debug.h"
 
-
-#define LIBAVCODEC_VER_AT_LEAST(major, minor) \
-	(LIBAVCODEC_VERSION_MAJOR > major || \
-	 (LIBAVCODEC_VERSION_MAJOR == major && \
-	  LIBAVCODEC_VERSION_MINOR >= minor))
-
-#if LIBAVCODEC_VER_AT_LEAST(51, 42)
-#define PIX_FMT_YUV420P AV_PIX_FMT_YUV420P
-#endif
-
-#if LIBAVCODEC_VER_AT_LEAST(56, 56)
-#ifndef CODEC_FLAG_GLOBAL_HEADER
-#define CODEC_FLAG_GLOBAL_HEADER AV_CODEC_FLAG_GLOBAL_HEADER
-#endif
-#ifndef FF_INPUT_BUFFER_PADDING_SIZE
-#define FF_INPUT_BUFFER_PADDING_SIZE AV_INPUT_BUFFER_PADDING_SIZE
-#endif
-#endif
-
-#if LIBAVCODEC_VER_AT_LEAST(57, 14)
-#define USE_CODECPAR
-#endif
-
-
 /* MP4 output */
 static AVFormatContext *fctx;
+#if LIBAVCODEC_VER_AT_LEAST(57, 25)
 static AVStream *vStream;
-#ifdef USE_CODECPAR
-static AVCodecContext *vEncoder;
 #endif
 static uint16_t max_width = 0, max_height = 0;
-int fps = 0;
+static int fps = 0;
 
+/* Supported target formats */
+static const char *janus_pp_av1_formats[] = {
+	"mp4", "mkv", NULL
+};
+const char **janus_pp_av1_get_extensions(void) {
+	return janus_pp_av1_formats;
+}
 
-int janus_pp_av1_create(char *destination, char *metadata, gboolean faststart) {
+/* Processing methods */
+int janus_pp_av1_create(char *destination, char *metadata, gboolean faststart, const char *extension) {
 	if(destination == NULL)
 		return -1;
 #if !LIBAVCODEC_VER_AT_LEAST(57, 25)
 	JANUS_LOG(LOG_ERR, "This version of libavcodec doesn't support AV1...\n");
 	return -1;
-#endif
-	/* Setup FFmpeg */
-#if ( LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58,9,100) )
-	av_register_all();
-#endif
-	/* Adjust logging to match the postprocessor's */
-	av_log_set_level(janus_log_level <= LOG_NONE ? AV_LOG_QUIET :
-		(janus_log_level == LOG_FATAL ? AV_LOG_FATAL :
-			(janus_log_level == LOG_ERR ? AV_LOG_ERROR :
-				(janus_log_level == LOG_WARN ? AV_LOG_WARNING :
-					(janus_log_level == LOG_INFO ? AV_LOG_INFO :
-						(janus_log_level == LOG_VERB ? AV_LOG_VERBOSE : AV_LOG_DEBUG))))));
-	/* MP4 output */
-	fctx = avformat_alloc_context();
+#else
+	/* .mkv is Matroska video */
+	if(!strcasecmp(extension, "mkv"))
+		extension = "matroska";
+
+	/* Video output */
+	fctx = janus_pp_create_avformatcontext(extension, metadata, destination);
 	if(fctx == NULL) {
 		JANUS_LOG(LOG_ERR, "Error allocating context\n");
 		return -1;
 	}
-	/* We save the metadata part as a comment (see #1189) */
-	if(metadata)
-		av_dict_set(&fctx->metadata, "comment", metadata, 0);
-	fctx->oformat = av_guess_format("mp4", NULL, NULL);
-	if(fctx->oformat == NULL) {
-		JANUS_LOG(LOG_ERR, "Error guessing format\n");
-		return -1;
-	}
-    char filename[1024];
-	snprintf(filename, sizeof(filename), "%s", destination);
-#ifdef USE_CODECPAR
-#if LIBAVCODEC_VER_AT_LEAST(57, 25)
-	AVCodec *codec = avcodec_find_encoder(AV_CODEC_ID_AV1);
-#else
-	if(!codec) {
-		/* Error opening video codec */
-		JANUS_LOG(LOG_ERR, "Encoder not available\n");
-		return -1;
-	}
-#endif
-	fctx->video_codec = codec;
-	fctx->oformat->video_codec = codec->id;
-	vStream = avformat_new_stream(fctx, codec);
-	vStream->id = fctx->nb_streams-1;
-	vEncoder = avcodec_alloc_context3(codec);
-	vEncoder->width = max_width;
-	vEncoder->height = max_height;
-	vEncoder->time_base = (AVRational){ 1, fps };
-	vEncoder->pix_fmt = AV_PIX_FMT_YUV420P;
-	vEncoder->flags |= CODEC_FLAG_GLOBAL_HEADER;
-	vEncoder->strict_std_compliance = -2;
-	if(avcodec_open2(vEncoder, codec, NULL) < 0) {
-		/* Error opening video codec */
-		JANUS_LOG(LOG_ERR, "Encoder error\n");
-		return -1;
-	}
-	avcodec_parameters_from_context(vStream->codecpar, vEncoder);
-#else
-	vStream = avformat_new_stream(fctx, 0);
+
+	vStream = janus_pp_new_video_avstream(fctx, AV_CODEC_ID_AV1, max_width, max_height);
 	if(vStream == NULL) {
 		JANUS_LOG(LOG_ERR, "Error adding stream\n");
 		return -1;
 	}
-#if LIBAVCODEC_VER_AT_LEAST(53, 21)
-	avcodec_get_context_defaults3(vStream->codec, AVMEDIA_TYPE_VIDEO);
-#else
-	avcodec_get_context_defaults2(vStream->codec, AVMEDIA_TYPE_VIDEO);
-#endif
-#if LIBAVCODEC_VER_AT_LEAST(57, 25)
-	vStream->codec->codec_id = AV_CODEC_ID_AV1;
-#endif
-	vStream->codec->codec_type = AVMEDIA_TYPE_VIDEO;
-	vStream->codec->time_base = (AVRational){1, fps};
-	vStream->time_base = (AVRational){1, 90000};
-	vStream->codec->width = max_width;
-	vStream->codec->height = max_height;
-	vStream->codec->pix_fmt = PIX_FMT_YUV420P;
-	//~ if (fctx->flags & AVFMT_GLOBALHEADER)
-		vStream->codec->flags |= CODEC_FLAG_GLOBAL_HEADER;
-#endif
+
 	AVDictionary *options = NULL;
 	if(faststart)
 		av_dict_set(&options, "movflags", "+faststart", 0);
 
-	int res = avio_open2(&fctx->pb, filename, AVIO_FLAG_WRITE, NULL, &options);
-	if(res < 0) {
-		JANUS_LOG(LOG_ERR, "Error opening file for output (%d)\n", res);
-		return -1;
-	}
 	if(avformat_write_header(fctx, &options) < 0) {
 		JANUS_LOG(LOG_ERR, "Error writing header\n");
 		return -1;
 	}
 	return 0;
+#endif
 }
 
 /* Helper to decode a leb128 integer  */
@@ -288,9 +205,11 @@ static void janus_pp_av1_parse_sh(char *buffer, uint16_t *width, uint16_t *heigh
 	*height = janus_pp_av1_getbits(base, fhbm1+1, &offset)+1;
 }
 
-int janus_pp_av1_preprocess(FILE *file, janus_pp_frame_packet *list) {
+int janus_pp_av1_preprocess(FILE *file, janus_pp_frame_packet *list, json_t *info) {
 	if(!file || !list)
 		return -1;
+	json_t *resolutions = NULL;
+	int last_width = -1, last_height = -1;
 	janus_pp_frame_packet *tmp = list;
 	int bytes = 0, min_ts_diff = 0, max_ts_diff = 0;
 	int rotation = -1;
@@ -372,6 +291,20 @@ int janus_pp_av1_preprocess(FILE *file, janus_pp_frame_packet *list) {
 					max_height = height;
 				}
 				JANUS_LOG(LOG_INFO, "  -- Detected new resolution: %"SCNu16"x%"SCNu16" (seq=%"SCNu16")\n", width, height, tmp->seq);
+				if(info && (last_width != width || last_height != height)) {
+					last_width = width;
+					last_height = height;
+					if(resolutions == NULL) {
+						resolutions = json_array();
+						json_object_set_new(info, "resolution", resolutions);
+					}
+					json_t *resolution = json_object();
+					double ts = (double)(tmp->ts-list->ts)/(double)90000;
+					json_object_set_new(resolution, "ts", json_real(ts));
+					json_object_set_new(resolution, "width", json_integer(width));
+					json_object_set_new(resolution, "height", json_integer(height));
+					json_array_append_new(resolutions, resolution);
+				}
 			}
 			payload += obusize;
 			len -= obusize;
@@ -384,7 +317,8 @@ int janus_pp_av1_preprocess(FILE *file, janus_pp_frame_packet *list) {
 		}
 		if(tmp->rotation != -1 && tmp->rotation != rotation) {
 			rotation = tmp->rotation;
-			JANUS_LOG(LOG_INFO, "Video rotation: %d degrees\n", rotation);
+			double ts = (double)(tmp->ts-list->ts)/(double)90000;
+			JANUS_LOG(LOG_INFO, "[%8.3fs] Video rotation: %d degrees\n", ts, rotation);
 		}
 		tmp = tmp->next;
 	}
@@ -422,6 +356,12 @@ int janus_pp_av1_process(FILE *file, janus_pp_frame_packet *list, int *working)
 	int len = 0, frameLen = 0, total = 0, dataLen = 0;
 	int keyFrame = 0;
 	gboolean keyframe_found = FALSE;
+#ifdef FF_API_INIT_PACKET
+	AVPacket *packet = av_packet_alloc();
+#else
+	AVPacket pkt = { 0 }, *packet = &pkt;
+#endif
+	AVRational timebase = {1, 90000};
 
 	while(*working && tmp != NULL) {
 		keyFrame = 0;
@@ -542,21 +482,23 @@ int janus_pp_av1_process(FILE *file, janus_pp_frame_packet *list, int *working)
 			total += frameLen;
 			JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Saving frame: %d (tot=%d)\n", tmp->ts, frameLen, total);
 
-			AVPacket packet;
-			av_init_packet(&packet);
-			packet.stream_index = 0;
-			packet.data = received_frame;
-			packet.size = frameLen;
+#ifdef FF_API_INIT_PACKET
+			av_packet_unref(packet);
+#else
+			av_init_packet(packet);
+#endif
+			packet->stream_index = 0;
+			packet->data = received_frame;
+			packet->size = frameLen;
 			if(keyFrame)
-				packet.flags |= AV_PKT_FLAG_KEY;
+				packet->flags |= AV_PKT_FLAG_KEY;
 
 			/* First we save to the file... */
-			packet.dts = tmp->ts-list->ts;
-			packet.pts = tmp->ts-list->ts;
+			packet->pts = packet->dts = av_rescale_q(tmp->ts-list->ts, timebase, fctx->streams[0]->time_base);
 			JANUS_LOG(LOG_HUGE, "%"SCNu64" - %"SCNu64" --> %"SCNu64"\n",
-				tmp->ts, list->ts, packet.pts);
+				tmp->ts, list->ts, packet->pts);
 			if(fctx) {
-				int res = av_write_frame(fctx, &packet);
+				int res = av_write_frame(fctx, packet);
 				if(res < 0) {
 					JANUS_LOG(LOG_ERR, "Error writing video frame to file... (error %d)\n", res);
 				}
@@ -564,6 +506,9 @@ int janus_pp_av1_process(FILE *file, janus_pp_frame_packet *list, int *working)
 		}
 		tmp = tmp->next;
 	}
+#ifdef FF_API_INIT_PACKET
+	av_packet_free(&packet);
+#endif
 	g_free(received_frame);
 	g_free(obu_data);
 	g_free(start);
@@ -572,23 +517,9 @@ int janus_pp_av1_process(FILE *file, janus_pp_frame_packet *list, int *working)
 
 /* Close MP4 file */
 void janus_pp_av1_close(void) {
-	if(fctx != NULL)
-		av_write_trailer(fctx);
-#ifdef USE_CODECPAR
-	if(vEncoder != NULL)
-		avcodec_close(vEncoder);
-#else
-	if(vStream != NULL && vStream->codec != NULL)
-		avcodec_close(vStream->codec);
-#endif
-	if(fctx != NULL && fctx->streams[0] != NULL) {
-#ifndef USE_CODECPAR
-		av_free(fctx->streams[0]->codec);
-#endif
-		av_free(fctx->streams[0]);
-	}
 	if(fctx != NULL) {
+		av_write_trailer(fctx);
 		avio_close(fctx->pb);
-		av_free(fctx);
+		avformat_free_context(fctx);
 	}
 }
