@@ -13,12 +13,14 @@
 #define JANUS_PP_AV1
 
 #include <stdio.h>
+#include <jansson.h>
 
 #include "pp-rtp.h"
 
 /* AV1 stuff */
-int janus_pp_av1_create(char *destination, char *metadata, gboolean faststart);
-int janus_pp_av1_preprocess(FILE *file, janus_pp_frame_packet *list);
+const char **janus_pp_av1_get_extensions(void);
+int janus_pp_av1_create(char *destination, char *metadata, gboolean faststart, const char *extension);
+int janus_pp_av1_preprocess(FILE *file, janus_pp_frame_packet *list, json_t *info);
 int janus_pp_av1_process(FILE *file, janus_pp_frame_packet *list, int *working);
 void janus_pp_av1_close(void);
 
