@@ -19,29 +19,17 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <libavcodec/avcodec.h>
-#include <libavformat/avformat.h>
 
+#include "pp-avformat.h"
 #include "pp-g722.h"
 #include "../debug.h"
 
-
-#define LIBAVCODEC_VER_AT_LEAST(major, minor) \
-	(LIBAVCODEC_VERSION_MAJOR > major || \
-	 (LIBAVCODEC_VERSION_MAJOR == major && \
-	  LIBAVCODEC_VERSION_MINOR >= minor))
-
-#if LIBAVCODEC_VER_AT_LEAST(57, 14)
-#define USE_CODECPAR
-#endif
-
-
 /* G.722 decoder */
 static AVCodec *dec_codec;			/* FFmpeg decoding codec */
 static AVCodecContext *dec_ctx;		/* FFmpeg decoding context */
 
 /* WAV header */
-typedef struct janus_pp_g711_wav {
+typedef struct janus_pp_g722_wav {
 	char riff[4];
 	uint32_t len;
 	char wave[4];
@@ -55,25 +43,22 @@ typedef struct janus_pp_g711_wav {
 	uint16_t channelbits;
 	char data[4];
 	uint32_t blocksize;
-} janus_pp_g711_wav;
+} janus_pp_g722_wav;
 static FILE *wav_file = NULL;
 
+/* Supported target formats */
+static const char *janus_pp_g722_formats[] = {
+	"wav", NULL
+};
+const char **janus_pp_g722_get_extensions(void) {
+	return janus_pp_g722_formats;
+}
 
 /* Processing methods */
 int janus_pp_g722_create(char *destination, char *metadata) {
 	if(destination == NULL)
 		return -1;
-	/* Setup FFmpeg */
-#if ( LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58,9,100) )
-	av_register_all();
-#endif
-	/* Adjust logging to match the postprocessor's */
-	av_log_set_level(janus_log_level <= LOG_NONE ? AV_LOG_QUIET :
-		(janus_log_level == LOG_FATAL ? AV_LOG_FATAL :
-			(janus_log_level == LOG_ERR ? AV_LOG_ERROR :
-				(janus_log_level == LOG_WARN ? AV_LOG_WARNING :
-					(janus_log_level == LOG_INFO ? AV_LOG_INFO :
-						(janus_log_level == LOG_VERB ? AV_LOG_VERBOSE : AV_LOG_DEBUG))))));
+	janus_pp_setup_avformat();
 	/* Create decoding context */
 #if LIBAVCODEC_VER_AT_LEAST(53, 21)
 	int codec = AV_CODEC_ID_ADPCM_G722;
@@ -105,7 +90,7 @@ int janus_pp_g722_create(char *destination, char *metadata) {
 	}
 	/* Add header */
 	JANUS_LOG(LOG_INFO, "Writing .wav file header\n");
-	janus_pp_g711_wav header = {
+	janus_pp_g722_wav header = {
 		{'R', 'I', 'F', 'F'},
 		0,
 		{'W', 'A', 'V', 'E'},
@@ -192,10 +177,14 @@ int janus_pp_g722_process(FILE *file, janus_pp_frame_packet *list, int *working)
 		JANUS_LOG(LOG_VERB, "Writing %d bytes out of %d (seq=%"SCNu16", step=%"SCNu16", ts=%"SCNu64", time=%"SCNu64"s)\n",
 			bytes, tmp->len, tmp->seq, diff, tmp->ts, (tmp->ts-list->ts)/8000);
 		/* Decode and save to wav */
-		AVPacket avpacket;
-		av_init_packet(&avpacket);
-		avpacket.data = (uint8_t *)buffer;
-		avpacket.size = bytes;
+#ifdef FF_API_INIT_PACKET
+		AVPacket *avpacket = av_packet_alloc();
+#else
+		AVPacket avpkt = { 0 }, *avpacket = &avpkt;
+		av_init_packet(avpacket);
+#endif
+		avpacket->data = (uint8_t *)buffer;
+		avpacket->size = bytes;
 		int err = 0;
 #if LIBAVCODEC_VER_AT_LEAST(55,28)
 		AVFrame *frame = av_frame_alloc();
@@ -203,7 +192,7 @@ int janus_pp_g722_process(FILE *file, janus_pp_frame_packet *list, int *working)
 		AVFrame *frame = avcodec_alloc_frame();
 #endif
 #ifdef USE_CODECPAR
-		err = avcodec_send_packet(dec_ctx, &avpacket);
+		err = avcodec_send_packet(dec_ctx, avpacket);
 		if(err < 0) {
 			JANUS_LOG(LOG_ERR, "Error decoding audio frame... (%d)\n", err);
 		} else {
@@ -212,7 +201,7 @@ int janus_pp_g722_process(FILE *file, janus_pp_frame_packet *list, int *working)
 		if(err > -1) {
 #else
 		int got_frame = 0;
-		err = avcodec_decode_audio4(dec_ctx, frame, &got_frame, &avpacket);
+		err = avcodec_decode_audio4(dec_ctx, frame, &got_frame, avpacket);
 		if(err < 0 || !got_frame) {
 			JANUS_LOG(LOG_ERR, "Error decoding audio frame... (%d)\n", err);
 		} else {
@@ -232,6 +221,9 @@ int janus_pp_g722_process(FILE *file, janus_pp_frame_packet *list, int *working)
 		av_frame_free(&frame);
 #else
 		avcodec_free_frame(&frame);
+#endif
+#ifdef FF_API_INIT_PACKET
+		av_packet_free(&avpacket);
 #endif
 		tmp = tmp->next;
 	}
