@@ -19,36 +19,10 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <libavcodec/avcodec.h>
-#include <libavformat/avformat.h>
-
+#include "pp-avformat.h"
 #include "pp-h265.h"
 #include "../debug.h"
 
-
-#define LIBAVCODEC_VER_AT_LEAST(major, minor) \
-	(LIBAVCODEC_VERSION_MAJOR > major || \
-	 (LIBAVCODEC_VERSION_MAJOR == major && \
-	  LIBAVCODEC_VERSION_MINOR >= minor))
-
-#if LIBAVCODEC_VER_AT_LEAST(51, 42)
-#define PIX_FMT_YUV420P AV_PIX_FMT_YUV420P
-#endif
-
-#if LIBAVCODEC_VER_AT_LEAST(56, 56)
-#ifndef CODEC_FLAG_GLOBAL_HEADER
-#define CODEC_FLAG_GLOBAL_HEADER AV_CODEC_FLAG_GLOBAL_HEADER
-#endif
-#ifndef FF_INPUT_BUFFER_PADDING_SIZE
-#define FF_INPUT_BUFFER_PADDING_SIZE AV_INPUT_BUFFER_PADDING_SIZE
-#endif
-#endif
-
-#if LIBAVCODEC_VER_AT_LEAST(57, 14)
-#define USE_CODECPAR
-#endif
-
-
 /* MP4 output */
 static AVFormatContext *fctx;
 static AVStream *vStream;
@@ -57,22 +31,26 @@ static AVCodecContext *vEncoder;
 #endif
 static int max_width = 0, max_height = 0, fps = 0;
 
+/* Supported target formats */
+static const char *janus_pp_h265_formats[] = {
+	"mp4", "mkv", NULL
+};
+const char **janus_pp_h265_get_extensions(void) {
+	return janus_pp_h265_formats;
+}
 
-int janus_pp_h265_create(char *destination, char *metadata, gboolean faststart) {
+/* Processing methods */
+int janus_pp_h265_create(char *destination, char *metadata, gboolean faststart, const char *extension) {
 	if(destination == NULL)
 		return -1;
-	/* Setup FFmpeg */
-#if ( LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58,9,100) )
-	av_register_all();
-#endif
-	/* Adjust logging to match the postprocessor's */
-	av_log_set_level(janus_log_level <= LOG_NONE ? AV_LOG_QUIET :
-		(janus_log_level == LOG_FATAL ? AV_LOG_FATAL :
-			(janus_log_level == LOG_ERR ? AV_LOG_ERROR :
-				(janus_log_level == LOG_WARN ? AV_LOG_WARNING :
-					(janus_log_level == LOG_INFO ? AV_LOG_INFO :
-						(janus_log_level == LOG_VERB ? AV_LOG_VERBOSE : AV_LOG_DEBUG))))));
-	/* MP4 output */
+
+	janus_pp_setup_avformat();
+
+	/* .mkv is Matroska video */
+	if(!strcasecmp(extension, "mkv"))
+		extension = "matroska";
+
+	/* Video output */
 	fctx = avformat_alloc_context();
 	if(fctx == NULL) {
 		JANUS_LOG(LOG_ERR, "Error allocating context\n");
@@ -81,7 +59,7 @@ int janus_pp_h265_create(char *destination, char *metadata, gboolean faststart)
 	/* We save the metadata part as a comment (see #1189) */
 	if(metadata)
 		av_dict_set(&fctx->metadata, "comment", metadata, 0);
-	fctx->oformat = av_guess_format("mp4", NULL, NULL);
+	fctx->oformat = av_guess_format(extension, NULL, NULL);
 	if(fctx->oformat == NULL) {
 		JANUS_LOG(LOG_ERR, "Error guessing format\n");
 		return -1;
@@ -96,7 +74,6 @@ int janus_pp_h265_create(char *destination, char *metadata, gboolean faststart)
 		return -1;
 	}
 	fctx->video_codec = codec;
-	fctx->oformat->video_codec = codec->id;
 	vStream = avformat_new_stream(fctx, codec);
 	vStream->id = fctx->nb_streams-1;
 	vEncoder = avcodec_alloc_context3(codec);
@@ -249,9 +226,11 @@ static void janus_pp_h265_parse_sps(char *buffer, int *width, int *height) {
 	*height = janus_pp_h265_eg_decode(base, &offset);
 }
 
-int janus_pp_h265_preprocess(FILE *file, janus_pp_frame_packet *list) {
+int janus_pp_h265_preprocess(FILE *file, janus_pp_frame_packet *list, json_t *info) {
 	if(!file || !list)
 		return -1;
+	json_t *resolutions = NULL;
+	int last_width = -1, last_height = -1;
 	janus_pp_frame_packet *tmp = list;
 	int bytes = 0, min_ts_diff = 0, max_ts_diff = 0;
 	int rotation = -1;
@@ -335,9 +314,94 @@ int janus_pp_h265_preprocess(FILE *file, janus_pp_frame_packet *list) {
 				max_width = width;
 				max_height = height;
 			}
+			if(info && (last_width != width || last_height != height)) {
+				last_width = width;
+				last_height = height;
+				if(resolutions == NULL) {
+					resolutions = json_array();
+					json_object_set_new(info, "resolution", resolutions);
+				}
+				json_t *resolution = json_object();
+				double ts = (double)(tmp->ts-list->ts)/(double)90000;
+				json_object_set_new(resolution, "ts", json_real(ts));
+				json_object_set_new(resolution, "width", json_integer(width));
+				json_object_set_new(resolution, "height", json_integer(height));
+				json_array_append_new(resolutions, resolution);
+			}
 		} else if(type == 34) {
 			/* PPS */
 			JANUS_LOG(LOG_HUGE, "[PPS] %u/%u/%u/%u\n", fbit, type, lid, tid);
+		} else if(type == 48) {
+			/* AP */
+			JANUS_LOG(LOG_HUGE, "[AP] %u/%u/%u/%u\n", fbit, type, lid, tid);
+			uint8_t *end = (uint8_t*)prebuffer + len, *p = NULL;
+			uint16_t payload_len;
+			for(p = (uint8_t*)prebuffer + 2; p < end - 2; p += payload_len) {
+				payload_len = (p[0] << 8 | p[1]);
+				p += 2;
+
+				uint16_t unit = 0;
+				memcpy(&unit, p, sizeof(uint16_t));
+				unit = ntohs(unit);
+				uint8_t fbit = (unit & 0x8000) >> 15;
+				uint8_t type = (unit & 0x7E00) >> 9;
+				uint8_t lid = (unit & 0x01F8) >> 3;
+				uint8_t tid = (unit & 0x0007);
+
+				switch(type) {
+					case 32:
+						JANUS_LOG(LOG_HUGE, "[VPS] %u/%u/%u/%u\n", fbit, type, lid, tid);
+						break;
+					case 33:
+						JANUS_LOG(LOG_HUGE, "[SPS] %u/%u/%u/%u\n", fbit, type, lid, tid);
+						/* Get rid of the Emulation Prevention code, if present */
+						int i = 0, j = 0, zeros = 0;
+						for(i=0; i<payload_len; i++) {
+							if(zeros == 2 && p[i] == 0x03) {
+								/* Found, get rid of it */
+								i++;
+								zeros = 0;
+							}
+							/* Update the content of the buffer as we go along */
+							p[j] = p[i];
+							if(p[i] == 0x00) {
+								/* Found a zero, may be part of a start code */
+								zeros++;
+							} else {
+								/* Not a zero, so not the beginning of a start code */
+								zeros = 0;
+							}
+							j++;
+						}
+						/* Parse to get width/height */
+						int width = 0, height = 0;
+						janus_pp_h265_parse_sps((char*)p, &width, &height);
+						if(width*height > max_width*max_height) {
+							max_width = width;
+							max_height = height;
+						}
+						if(info && (last_width != width || last_height != height)) {
+							last_width = width;
+							last_height = height;
+							if(resolutions == NULL) {
+								resolutions = json_array();
+								json_object_set_new(info, "resolution", resolutions);
+							}
+							json_t *resolution = json_object();
+							double ts = (double)(tmp->ts-list->ts)/(double)90000;
+							json_object_set_new(resolution, "ts", json_real(ts));
+							json_object_set_new(resolution, "width", json_integer(width));
+							json_object_set_new(resolution, "height", json_integer(height));
+							json_array_append_new(resolutions, resolution);
+						}
+						break;
+					case 34:
+						JANUS_LOG(LOG_HUGE, "[PPS] %u/%u/%u/%u\n", fbit, type, lid, tid);
+						break;
+					default:
+						break;
+				}
+			}
 		} else if(type == 49) {
 			/* FU */
 			uint8_t fuh = prebuffer[2];
@@ -354,7 +418,8 @@ int janus_pp_h265_preprocess(FILE *file, janus_pp_frame_packet *list) {
 		}
 		if(tmp->rotation != -1 && tmp->rotation != rotation) {
 			rotation = tmp->rotation;
-			JANUS_LOG(LOG_INFO, "Video rotation: %d degrees\n", rotation);
+			double ts = (double)(tmp->ts-list->ts)/(double)90000;
+			JANUS_LOG(LOG_INFO, "[%8.3fs] Video rotation: %d degrees\n", ts, rotation);
 		}
 		tmp = tmp->next;
 	}
@@ -392,6 +457,12 @@ int janus_pp_h265_process(FILE *file, janus_pp_frame_packet *list, int *working)
 	int len = 0, frameLen = 0;
 	int keyFrame = 0;
 	gboolean keyframe_found = FALSE;
+#ifdef FF_API_INIT_PACKET
+	AVPacket *packet = av_packet_alloc();
+#else
+	AVPacket pkt = { 0 }, *packet = &pkt;
+#endif
+	AVRational timebase = {1, 90000};
 
 	while(*working && tmp != NULL) {
 		keyFrame = 0;
@@ -450,8 +521,38 @@ int janus_pp_h265_process(FILE *file, janus_pp_frame_packet *list, int *working)
 				memset(temp + 1, 0x00, 1);
 				memset(temp + 2, 0x01, 1);
 				frameLen += 3;
-			}
-			if(type == 49) {
+			} else if(type == 48) {
+				/* AP */
+				uint8_t *end = buffer + len, *p = NULL;
+				uint16_t payload_len;
+				for(p = buffer + 2; p < end - 2; p += payload_len) {
+					payload_len = (p[0] << 8 | p[1]);
+					p += 2;
+					uint8_t *temp = received_frame + frameLen;
+					temp[0] = 0;
+					temp[1] = 0;
+					temp[2] = 1;
+					memcpy(temp + 3, p, payload_len);
+					switch(p[0] >> 1) {
+						case 32:
+						case 33:
+						case 34:
+							keyFrame = 1;
+							if(!keyframe_found) {
+								keyframe_found = TRUE;
+								JANUS_LOG(LOG_INFO, "First keyframe: %"SCNu64"\n", tmp->ts-list->ts);
+							}
+							break;
+						default:
+							break;
+					}
+					frameLen += 3 + payload_len;
+				}
+				if(tmp->next == NULL || tmp->next->ts > tmp->ts)
+					break;
+				tmp = tmp->next;
+				continue;
+			} else if(type == 49) {
 				/* Check if this is the beginning of the FU */
 				uint8_t fuh = *(buffer+2);
 				uint8_t startbit = (fuh & 0x80) >> 7;
@@ -490,21 +591,23 @@ int janus_pp_h265_process(FILE *file, janus_pp_frame_packet *list, int *working)
 			/* Save the frame */
 			memset(received_frame + frameLen, 0, FF_INPUT_BUFFER_PADDING_SIZE);
 
-			AVPacket packet;
-			av_init_packet(&packet);
-			packet.stream_index = 0;
-			packet.data = received_frame;
-			packet.size = frameLen;
+#ifdef FF_API_INIT_PACKET
+			av_packet_unref(packet);
+#else
+			av_init_packet(packet);
+#endif
+			packet->stream_index = 0;
+			packet->data = received_frame;
+			packet->size = frameLen;
 			if(keyFrame)
-				packet.flags |= AV_PKT_FLAG_KEY;
+				packet->flags |= AV_PKT_FLAG_KEY;
 
 			/* First we save to the file... */
-			packet.dts = tmp->ts-list->ts;
-			packet.pts = tmp->ts-list->ts;
+			packet->pts = packet->dts = av_rescale_q(tmp->ts-list->ts, timebase, fctx->streams[0]->time_base);
 			JANUS_LOG(LOG_HUGE, "%"SCNu64" - %"SCNu64" --> %"SCNu64"\n",
-				tmp->ts, list->ts, packet.pts);
+				tmp->ts, list->ts, packet->pts);
 			if(fctx) {
-				int res = av_write_frame(fctx, &packet);
+				int res = av_write_frame(fctx, packet);
 				if(res < 0) {
 					JANUS_LOG(LOG_ERR, "Error writing video frame to file... (error %d)\n", res);
 				}
@@ -512,6 +615,9 @@ int janus_pp_h265_process(FILE *file, janus_pp_frame_packet *list, int *working)
 		}
 		tmp = tmp->next;
 	}
+#ifdef FF_API_INIT_PACKET
+	av_packet_free(&packet);
+#endif
 	g_free(received_frame);
 	g_free(start);
 	return 0;
