@@ -19,13 +19,10 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <libavcodec/avcodec.h>
-#include <libavformat/avformat.h>
-
+#include "pp-avformat.h"
 #include "pp-webm.h"
 #include "../debug.h"
 
-
 /* WebRTC stuff (VP8/VP9) */
 #if defined(__ppc__) || defined(__ppc64__)
 	# define swap2(d)  \
@@ -35,38 +32,22 @@
 	# define swap2(d) d
 #endif
 
-#define LIBAVCODEC_VER_AT_LEAST(major, minor) \
-	(LIBAVCODEC_VERSION_MAJOR > major || \
-	 (LIBAVCODEC_VERSION_MAJOR == major && \
-	  LIBAVCODEC_VERSION_MINOR >= minor))
-
-#if LIBAVCODEC_VER_AT_LEAST(51, 42)
-#define PIX_FMT_YUV420P AV_PIX_FMT_YUV420P
-#endif
-
-#if LIBAVCODEC_VER_AT_LEAST(56, 56)
-#ifndef CODEC_FLAG_GLOBAL_HEADER
-#define CODEC_FLAG_GLOBAL_HEADER AV_CODEC_FLAG_GLOBAL_HEADER
-#endif
-#ifndef FF_INPUT_BUFFER_PADDING_SIZE
-#define FF_INPUT_BUFFER_PADDING_SIZE AV_INPUT_BUFFER_PADDING_SIZE
-#endif
-#endif
-
-#if LIBAVCODEC_VER_AT_LEAST(57, 14)
-#define USE_CODECPAR
-#endif
-
 
 /* WebM output */
 static AVFormatContext *fctx;
 static AVStream *vStream;
-#ifdef USE_CODECPAR
-static AVCodecContext *vEncoder;
-#endif
 static int max_width = 0, max_height = 0, fps = 0;
 
-int janus_pp_webm_create(char *destination, char *metadata, gboolean vp8) {
+/* Supported target formats */
+static const char *janus_pp_webm_formats[] = {
+	"webm", "mkv", NULL
+};
+const char **janus_pp_webm_get_extensions(void) {
+	return janus_pp_webm_formats;
+}
+
+/* Processing methods */
+int janus_pp_webm_create(char *destination, char *metadata, gboolean vp8, const char *extension) {
 	if(destination == NULL)
 		return -1;
 #if LIBAVCODEC_VERSION_MAJOR < 55
@@ -75,92 +56,34 @@ int janus_pp_webm_create(char *destination, char *metadata, gboolean vp8) {
 		return -1;
 	}
 #endif
-	/* Setup FFmpeg */
-#if ( LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58,9,100) )
-	av_register_all();
-#endif
-	/* Adjust logging to match the postprocessor's */
-	av_log_set_level(janus_log_level <= LOG_NONE ? AV_LOG_QUIET :
-		(janus_log_level == LOG_FATAL ? AV_LOG_FATAL :
-			(janus_log_level == LOG_ERR ? AV_LOG_ERROR :
-				(janus_log_level == LOG_WARN ? AV_LOG_WARNING :
-					(janus_log_level == LOG_INFO ? AV_LOG_INFO :
-						(janus_log_level == LOG_VERB ? AV_LOG_VERBOSE : AV_LOG_DEBUG))))));
-	/* WebM output */
-	fctx = avformat_alloc_context();
+	/* .mkv is Matroska video */
+	if(!strcasecmp(extension, "mkv"))
+		extension = "matroska";
+
+	/* Video output */
+	fctx = janus_pp_create_avformatcontext(extension, metadata, destination);
 	if(fctx == NULL) {
 		JANUS_LOG(LOG_ERR, "Error allocating context\n");
 		return -1;
 	}
-	/* We save the metadata part as a comment (see #1189) */
-	if(metadata)
-		av_dict_set(&fctx->metadata, "comment", metadata, 0);
-	fctx->oformat = av_guess_format("webm", NULL, NULL);
-	if(fctx->oformat == NULL) {
-		JANUS_LOG(LOG_ERR, "Error guessing format\n");
-		return -1;
-	}
-    char filename[1024];
-	snprintf(filename, sizeof(filename), "%s", destination);
-#ifdef USE_CODECPAR
-	AVCodec *codec = avcodec_find_encoder(vp8 ? AV_CODEC_ID_VP8 : AV_CODEC_ID_VP9);
-	if(!codec) {
-		/* Error opening video codec */
-		JANUS_LOG(LOG_ERR, "Encoder not available\n");
-		return -1;
-	}
-	fctx->video_codec = codec;
-	fctx->oformat->video_codec = codec->id;
-	vStream = avformat_new_stream(fctx, codec);
-	vStream->id = fctx->nb_streams-1;
-	vEncoder = avcodec_alloc_context3(codec);
-	vEncoder->width = max_width;
-	vEncoder->height = max_height;
-	vEncoder->time_base = (AVRational){ 1, fps };
-	vEncoder->pix_fmt = AV_PIX_FMT_YUV420P;
-	vEncoder->flags |= CODEC_FLAG_GLOBAL_HEADER;
-	if(avcodec_open2(vEncoder, codec, NULL) < 0) {
-		/* Error opening video codec */
-		JANUS_LOG(LOG_ERR, "Encoder error\n");
-		return -1;
-	}
-	avcodec_parameters_from_context(vStream->codecpar, vEncoder);
+
+	int codec_id;
+#if LIBAVCODEC_VER_AT_LEAST(54, 25)
+	#if LIBAVCODEC_VERSION_MAJOR >= 55
+	codec_id = vp8 ? AV_CODEC_ID_VP8 : AV_CODEC_ID_VP9;
+	#else
+	codec_id = AV_CODEC_ID_VP8;
+	#endif
 #else
-	//~ vStream = av_new_stream(fctx, 0);
-	vStream = avformat_new_stream(fctx, 0);
+	codec_id = CODEC_ID_VP8;
+#endif
+
+	vStream = janus_pp_new_video_avstream(fctx, codec_id, max_width, max_height);
 	if(vStream == NULL) {
 		JANUS_LOG(LOG_ERR, "Error adding stream\n");
 		return -1;
 	}
-	//~ avcodec_get_context_defaults2(vStream->codec, CODEC_TYPE_VIDEO);
-#if LIBAVCODEC_VER_AT_LEAST(53, 21)
-	avcodec_get_context_defaults3(vStream->codec, AVMEDIA_TYPE_VIDEO);
-#else
-	avcodec_get_context_defaults2(vStream->codec, AVMEDIA_TYPE_VIDEO);
-#endif
-#if LIBAVCODEC_VER_AT_LEAST(54, 25)
-	#if LIBAVCODEC_VERSION_MAJOR >= 55
-	vStream->codec->codec_id = vp8 ? AV_CODEC_ID_VP8 : AV_CODEC_ID_VP9;
-	#else
-	vStream->codec->codec_id = AV_CODEC_ID_VP8;
-	#endif
-#else
-	vStream->codec->codec_id = CODEC_ID_VP8;
-#endif
-	//~ vStream->codec->codec_type = CODEC_TYPE_VIDEO;
-	vStream->codec->codec_type = AVMEDIA_TYPE_VIDEO;
-	vStream->codec->time_base = (AVRational){1, fps};
-	vStream->codec->width = max_width;
-	vStream->codec->height = max_height;
-	vStream->codec->pix_fmt = PIX_FMT_YUV420P;
-	if (fctx->flags & AVFMT_GLOBALHEADER)
-		vStream->codec->flags |= CODEC_FLAG_GLOBAL_HEADER;
-#endif
-	int res = avio_open(&fctx->pb, filename, AVIO_FLAG_WRITE);
-	if(res < 0) {
-		JANUS_LOG(LOG_ERR, "Error opening file for output (%d)\n", res);
-		return -1;
-	}
+
 	if(avformat_write_header(fctx, NULL) < 0) {
 		JANUS_LOG(LOG_ERR, "Error writing header\n");
 		return -1;
@@ -168,9 +91,11 @@ int janus_pp_webm_create(char *destination, char *metadata, gboolean vp8) {
 	return 0;
 }
 
-int janus_pp_webm_preprocess(FILE *file, janus_pp_frame_packet *list, gboolean vp8) {
+int janus_pp_webm_preprocess(FILE *file, janus_pp_frame_packet *list, gboolean vp8, json_t *info) {
 	if(!file || !list)
 		return -1;
+	json_t *resolutions = NULL;
+	int last_width = -1, last_height = -1;
 	janus_pp_frame_packet *tmp = list;
 	int bytes = 0, min_ts_diff = 0, max_ts_diff = 0;
 	int rotation = -1;
@@ -198,7 +123,8 @@ int janus_pp_webm_preprocess(FILE *file, janus_pp_frame_packet *list, gboolean v
 		}
 		if(tmp->rotation != -1 && tmp->rotation != rotation) {
 			rotation = tmp->rotation;
-			JANUS_LOG(LOG_INFO, "Video rotation: %d degrees\n", rotation);
+			double ts = (double)(tmp->ts-list->ts)/(double)90000;
+			JANUS_LOG(LOG_INFO, "[%8.3fs] Video rotation: %d degrees\n", ts, rotation);
 		}
 		if(vp8) {
 			/* https://tools.ietf.org/html/draft-ietf-payload-vp8 */
@@ -268,6 +194,20 @@ int janus_pp_webm_preprocess(FILE *file, janus_pp_frame_packet *list, gboolean v
 							max_width = vp8w;
 							max_height = vp8h;
 						}
+						if(info && (last_width != vp8w || last_height != vp8h)) {
+							last_width = vp8w;
+							last_height = vp8h;
+							if(resolutions == NULL) {
+								resolutions = json_array();
+								json_object_set_new(info, "resolution", resolutions);
+							}
+							json_t *resolution = json_object();
+							double ts = (double)(tmp->ts-list->ts)/(double)90000;
+							json_object_set_new(resolution, "ts", json_real(ts));
+							json_object_set_new(resolution, "width", json_integer(vp8w));
+							json_object_set_new(resolution, "height", json_integer(vp8h));
+							json_array_append_new(resolutions, resolution);
+						}
 					}
 				}
 			}
@@ -342,6 +282,20 @@ int janus_pp_webm_preprocess(FILE *file, janus_pp_frame_packet *list, gboolean v
 							max_width = width;
 							max_height = height;
 						}
+						if(info && (last_width != width || last_height != height)) {
+							last_width = width;
+							last_height = height;
+									if(resolutions == NULL) {
+								resolutions = json_array();
+								json_object_set_new(info, "resolution", resolutions);
+							}
+							json_t *resolution = json_object();
+							double ts = (double)(tmp->ts-list->ts)/(double)90000;
+							json_object_set_new(resolution, "ts", json_real(ts));
+							json_object_set_new(resolution, "width", json_integer(width));
+							json_object_set_new(resolution, "height", json_integer(height));
+							json_array_append_new(resolutions, resolution);
+						}
 					}
 				}
 			}
@@ -374,6 +328,12 @@ int janus_pp_webm_process(FILE *file, janus_pp_frame_packet *list, gboolean vp8,
 	int len = 0, frameLen = 0;
 	int keyFrame = 0;
 	gboolean keyframe_found = FALSE;
+#ifdef FF_API_INIT_PACKET
+	AVPacket *packet = av_packet_alloc();
+#else
+	AVPacket pkt = { 0 }, *packet = &pkt;
+#endif
+	AVRational timebase = {1, 90000};
 
 	while(*working && tmp != NULL) {
 		keyFrame = 0;
@@ -548,6 +508,7 @@ int janus_pp_webm_process(FILE *file, janus_pp_frame_packet *list, gboolean vp8,
 					uint8_t ybit = (vp9pd & 0x10);
 					uint8_t gbit = (vp9pd & 0x08);
 					if(ybit) {
+						keyFrame = 1;
 						/* Iterate on all spatial layers and get resolution */
 						buffer++;
 						len--;
@@ -609,28 +570,31 @@ int janus_pp_webm_process(FILE *file, janus_pp_frame_packet *list, gboolean vp8,
 		if(frameLen > 0) {
 			memset(received_frame + frameLen, 0, FF_INPUT_BUFFER_PADDING_SIZE);
 
-			AVPacket packet;
-			av_init_packet(&packet);
-			packet.stream_index = 0;
-			packet.data = received_frame;
-			packet.size = frameLen;
+#ifdef FF_API_INIT_PACKET
+			av_packet_unref(packet);
+#else
+			av_init_packet(packet);
+#endif
+			packet->stream_index = 0;
+			packet->data = received_frame;
+			packet->size = frameLen;
 			if(keyFrame)
 				//~ packet.flags |= PKT_FLAG_KEY;
-				packet.flags |= AV_PKT_FLAG_KEY;
+				packet->flags |= AV_PKT_FLAG_KEY;
 
 			/* First we save to the file... */
-			//~ packet.dts = AV_NOPTS_VALUE;
-			//~ packet.pts = AV_NOPTS_VALUE;
-			packet.dts = (tmp->ts-list->ts)/90;
-			packet.pts = (tmp->ts-list->ts)/90;
+			packet->pts = packet->dts = av_rescale_q(tmp->ts-list->ts, timebase, fctx->streams[0]->time_base);
 			if(fctx) {
-				if(av_write_frame(fctx, &packet) < 0) {
+				if(av_write_frame(fctx, packet) < 0) {
 					JANUS_LOG(LOG_ERR, "Error writing video frame to file...\n");
 				}
 			}
 		}
 		tmp = tmp->next;
 	}
+#ifdef FF_API_INIT_PACKET
+	av_packet_free(&packet);
+#endif
 	g_free(received_frame);
 	g_free(start);
 	return 0;
@@ -638,24 +602,9 @@ int janus_pp_webm_process(FILE *file, janus_pp_frame_packet *list, gboolean vp8,
 
 /* Close WebM file */
 void janus_pp_webm_close(void) {
-	if(fctx != NULL)
-		av_write_trailer(fctx);
-#ifdef USE_CODECPAR
-	if(vEncoder != NULL)
-		avcodec_close(vEncoder);
-#else
-	if(vStream != NULL && vStream->codec != NULL)
-		avcodec_close(vStream->codec);
-#endif
-	if(fctx != NULL && fctx->streams[0] != NULL) {
-#ifndef USE_CODECPAR
-		av_free(fctx->streams[0]->codec);
-#endif
-		av_free(fctx->streams[0]);
-	}
 	if(fctx != NULL) {
-		//~ url_fclose(fctx->pb);
+		av_write_trailer(fctx);
 		avio_close(fctx->pb);
-		av_free(fctx);
+		avformat_free_context(fctx);
 	}
 }
