@@ -180,6 +180,7 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean rids_hml
 						stream->audio_rtcp_ctx->tb = 48000;	/* May change later */
 					}
 				}
+				gboolean receiving = (stream->audio_recv == TRUE);
 				switch(m->direction) {
 					case JANUS_SDP_INACTIVE:
 					case JANUS_SDP_INVALID:
@@ -203,6 +204,8 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean rids_hml
 						stream->audio_recv = TRUE;
 						break;
 				}
+				if(receiving != stream->audio_recv)
+					janus_ice_notify_media_stopped(handle);
 				if(m->ptypes != NULL) {
 					g_list_free(stream->audio_payload_types);
 					stream->audio_payload_types = g_list_copy(m->ptypes);
@@ -230,6 +233,7 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean rids_hml
 						stream->video_rtcp_ctx[0]->tb = 90000;	/* May change later */
 					}
 				}
+				gboolean receiving = (stream->video_recv == TRUE);
 				switch(m->direction) {
 					case JANUS_SDP_INACTIVE:
 					case JANUS_SDP_INVALID:
@@ -253,6 +257,8 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean rids_hml
 						stream->video_recv = TRUE;
 						break;
 				}
+				if(receiving != stream->video_recv)
+					janus_ice_notify_media_stopped(handle);
 				if(m->ptypes != NULL) {
 					g_list_free(stream->video_payload_types);
 					stream->video_payload_types = g_list_copy(m->ptypes);
@@ -571,6 +577,9 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean rids_hml
 									if(stream->clock_rates == NULL)
 										stream->clock_rates = g_hash_table_new(NULL, NULL);
 									g_hash_table_insert(stream->clock_rates, GINT_TO_POINTER(ptype), GUINT_TO_POINTER(clock_rate));
+									/* Check if opus/red  is negotiated */
+									if(strstr(a->value, "red/48000/2"))
+										stream->opusred_pt = ptype;
 								}
 							}
 						}
@@ -1261,14 +1270,14 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 			if(audio == 1 && m->port > 0) {
 				g_snprintf(buffer_part, sizeof(buffer_part),
 					" %s", handle->audio_mid ? handle->audio_mid : "audio");
-				g_strlcat(buffer, buffer_part, sizeof(buffer));
+				janus_strlcat(buffer, buffer_part, sizeof(buffer));
 			}
 		} else if(m->type == JANUS_SDP_VIDEO) {
 			video++;
 			if(video == 1 && m->port > 0) {
 				g_snprintf(buffer_part, sizeof(buffer_part),
 					" %s", handle->video_mid ? handle->video_mid : "video");
-				g_strlcat(buffer, buffer_part, sizeof(buffer));
+				janus_strlcat(buffer, buffer_part, sizeof(buffer));
 			}
 #ifdef HAVE_SCTP
 		} else if(m->type == JANUS_SDP_APPLICATION) {
@@ -1277,7 +1286,7 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 			if(data == 1 && m->port > 0) {
 				g_snprintf(buffer_part, sizeof(buffer_part),
 					" %s", handle->data_mid ? handle->data_mid : "data");
-				g_strlcat(buffer, buffer_part, sizeof(buffer));
+				janus_strlcat(buffer, buffer_part, sizeof(buffer));
 			}
 #endif
 		}
@@ -1287,6 +1296,10 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 	GList *first = anon->attributes;
 	janus_sdp_attribute *a = janus_sdp_attribute_create("group", "%s", buffer);
 	anon->attributes = g_list_insert_before(anon->attributes, first, a);
+	/* Notify we support 1-byte and 2-byte extensions
+	 * FIXME We should actually negotiate this, in the future */
+	a = janus_sdp_attribute_create("extmap-allow-mixed", NULL);
+	anon->attributes = g_list_insert_before(anon->attributes, first, a);
 	/* msid-semantic: add new global attribute */
 	a = janus_sdp_attribute_create("msid-semantic", " WMS janus");
 	anon->attributes = g_list_insert_before(anon->attributes, first, a);
@@ -1328,6 +1341,7 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 				stream->audio_ssrc = 0;
 			}
 			if(audio == 1) {
+				gboolean receiving = (stream->audio_recv == TRUE);
 				switch(m->direction) {
 					case JANUS_SDP_INACTIVE:
 						stream->audio_send = FALSE;
@@ -1348,6 +1362,8 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 						stream->audio_recv = TRUE;
 						break;
 				}
+				if(receiving != stream->audio_recv)
+					janus_ice_notify_media_stopped(handle);
 			}
 		} else if(m->type == JANUS_SDP_VIDEO) {
 			video++;
@@ -1361,6 +1377,7 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 				stream->video_ssrc = 0;
 			}
 			if(video == 1) {
+				gboolean receiving = (stream->video_recv == TRUE);
 				switch(m->direction) {
 					case JANUS_SDP_INACTIVE:
 						stream->video_send = FALSE;
@@ -1381,6 +1398,8 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 						stream->video_recv = TRUE;
 						break;
 				}
+				if(receiving != stream->video_recv)
+					janus_ice_notify_media_stopped(handle);
 				if(janus_flags_is_set(&handle->webrtc_flags, JANUS_ICE_HANDLE_WEBRTC_RFC4588_RTX)) {
 					/* Add RFC4588 stuff */
 					if(stream->rtx_payload_types && g_hash_table_size(stream->rtx_payload_types) > 0) {
@@ -1528,10 +1547,10 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 				a = janus_sdp_attribute_create("rid", "%s recv", stream->rid[index]);
 				m->attributes = g_list_append(m->attributes, a);
 				if(strlen(rids) == 0) {
-					g_strlcat(rids, stream->rid[index], sizeof(rids));
+					janus_strlcat(rids, stream->rid[index], sizeof(rids));
 				} else {
-					g_strlcat(rids, ";", sizeof(rids));
-					g_strlcat(rids, stream->rid[index], sizeof(rids));
+					janus_strlcat(rids, ";", sizeof(rids));
+					janus_strlcat(rids, stream->rid[index], sizeof(rids));
 				}
 			}
 			if(stream->legacy_rid) {
