@@ -49,6 +49,23 @@ gint64 janus_get_real_time(void);
  * @returns A pointer to the updated text string (re-allocated or just updated) */
 char *janus_string_replace(char *message, const char *old_string, const char *new_string) G_GNUC_WARN_UNUSED_RESULT;
 
+/*! \brief Helper method to concatenate strings and log an error if truncation occured
+ * @param[in] dest Destination buffer, already containing one nul-terminated string
+ * @param[in] src Source buffer
+ * @param[in] dest_size Length of dest buffer in bytes (not length of existing string inside dest)
+ * @returns Size of attempted result, if retval >= dest_size, truncation occurred (and an error will be logged). */
+size_t janus_strlcat(char *dest, const char *src, size_t dest_size);
+
+/*! \brief Alternative helper method to concatenate strings and log an error if truncation occured,
+ * which uses memccpy instead of g_strlcat and so is supposed to be faster
+ * @note The offset attribute is input/output, and updated any time the method is called
+ * @param[in] dest Destination buffer, already containing one nul-terminated string
+ * @param[in] src Source buffer
+ * @param[in] dest_size Length of dest buffer in bytes (not length of existing string inside dest)
+ * @param[in] offset Offset of where to start appending, in the destination buffer
+ * @returns 0 in case of success, a negative integer otherwise */
+int janus_strlcat_fast(char *dest, const char *src, size_t dest_size, size_t *offset);
+
 /*! \brief Helper to parse yes/no|true/false configuration values
  * @param value The configuration value to parse
  * @returns true if the value contains a "yes", "YES", "true", TRUE", "1", false otherwise */
@@ -61,16 +78,37 @@ gboolean janus_is_true(const char *value);
 gboolean janus_strcmp_const_time(const void *str1, const void *str2);
 
 /*! \brief Helper to generate random 32-bit unsigned integers (useful for SSRCs, etc.)
- * @note Currently just wraps g_random_int()
- * @returns A random 32-bit unsigned integer */
+ * @note Warning: this will fall back to a non-cryptographically safe PRNG in case
+ * the crypto library RAND_bytes() call fails.
+ * @returns A (mostly crypto-safe) random 32-bit unsigned integer */
 guint32 janus_random_uint32(void);
 
-/*! \brief Helper to generate random 64-bit unsigned integers (useful for Janus IDs)
- * @returns A random 64-bit unsigned integer */
+/*! \brief Helper to generate random 64-bit unsigned integers
+ * @note Unlike janus_random_uint64(), which actually only generates 52 bits, this
+ * generates the full 64 bits. See the janus_random_uint64() docstring for details.
+ * Warning: this will fall back to a non-cryptographically safe PRNG in case
+ * the crypto library RAND_bytes() call fails.
+ * @returns A (mostly crypto-safe) random 52-bit unsigned integer */
+guint64 janus_random_uint64_full(void);
+
+/*! \brief Helper to generate random 52 bit unsigned integers
+ * @note The reason for 52 instead of 64 bits: Javascript does not have real integers,
+ * its builtin "number" type is a float64. Thus, only integer values up to
+ * <code>Number.MAX_SAFE_INTEGER == 2^53 - 1 == 9007199254740991</code>
+ * can be safely represented in Javascript. This method returns such numbers.
+ * Use this method instead of janus_random_uint64_full() whenever you generate numbers which
+ * might end up in Javascript (via JSON API).
+ * This method is called janus_random_uint64() instead of janus_random_uint52() (or similar)
+ * for backwards compatibility.
+ * Warning: this will fall back to a non-cryptographically safe PRNG in case
+ * the crypto library RAND_bytes() call fails.
+ * @returns A (mostly crypto-safe) random 64-bit unsigned integer */
 guint64 janus_random_uint64(void);
 
 /*! \brief Helper to generate random UUIDs (needed by some plugins)
- * @returns A random UUID string, which must be deallocated with \c g_free */
+ * Warning: this will fall back to a non-cryptographically safe PRNG in case
+ * the crypto library RAND_bytes() call fails.
+ * @returns A (mostly crypto-safe) random UUID string, which must be deallocated with \c g_free */
 char *janus_random_uuid(void);
 
 /*! \brief Helper to generate an allocated copy of a guint64 number
@@ -84,7 +122,7 @@ guint64 *janus_uint64_dup(guint64 num);
 
 /*! \brief Helper to hash a guint64 number to another guint64 number
  * @note We currently only use for event handlers
- * @param
+ * @param num The guint64 number to hash
  * @returns The hashed number */
 guint64 janus_uint64_hash(guint64 num);
 
@@ -349,6 +387,32 @@ typedef struct janus_vp9_svc_info {
  * @returns 0 in case of success, a negative integer otherwise */
 int janus_vp9_parse_svc(char *buffer, int len, gboolean *found, janus_vp9_svc_info *info);
 
+/*! \brief Helper struct to address a specific RED block */
+typedef struct janus_red_block {
+	uint8_t pt;
+	uint32_t ts_offset;
+	uint8_t *data;
+	uint16_t length;
+} janus_red_block;
+/*! \brief Helper method to parse an RTP payload to return a list of RED blocks
+ * \note The returned list is owned by the caller, and must be freed
+ * @param[in] buffer The RTP payload to process
+ * @param[in] len The length of the RTP payload
+ * @returns An allocated GList of janus_red_block, if successful, NULL otherwise */
+GList *janus_red_parse_blocks(char *buffer, int len);
+/*! \brief Helper method to pack multiple buffers in a RED payload
+ * @param[in] buffer The RTP payload to write to
+ * @param[in] len The size of the RTP payload buffer
+ * @param[in] blocks Linked list of janus_red_block instances to add to the payload
+ * @returns The size of the RED payload in case of success, a negative integer otherwise */
+int janus_red_pack_blocks(char *buffer, int len, GList *blocks);
+/*! \brief Helper method to overwrite all RTP payload types in RED blocks
+ * @param[in] buffer The RED block payload to update
+ * @param[in] len The size of the payload buffer
+ * @param[in] pt The RTP payload type to set in all blocks
+ * @returns 0 in case of success, a negative integer otherwise */
+int janus_red_replace_block_pt(char *buffer, int len, int pt);
+
 /*! \brief Helper method to push individual bits at the end of a word
  * @param[in] word Initial value of word
  * @param[in] num Number of bits to push
@@ -384,6 +448,18 @@ void janus_set3(guint8 *data, size_t i, guint32 val);
  */
 void janus_set4(guint8 *data, size_t i, guint32 val);
 
+/* \brief Helpers to read a bit from a bitstream
+ * @param[in] base Pointer to the start of the bitstream
+ * @param[in] offset Offset in bits from the start
+ * @returns The value of the bit */
+uint8_t janus_bitstream_getbit(uint8_t *base, uint32_t offset);
+/* \brief Helpers to read agroup of bits from a bitstream
+ * @param[in] base Pointer to the start of the bitstream
+ * @param[in] num The number of bits to read
+ * @param[in] offset Offset in bits from the start
+ * @returns The value of the bits */
+uint32_t janus_bitstream_getbits(uint8_t *base, uint8_t num, uint32_t *offset);
+
 /*! \brief Helper method to compress a string to gzip (using zlib)
  * \note It's up to you to provide a buffer large enough for the compressed
  * data: in case the buffer isn't large enough, the request will fail
